# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports
import re
from types import *
from urllib import quote as url_quote, unquote as url_unquote

# zope class imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view
from AccessControl.ZopeGuards import guarded_getattr
from DocumentTemplate.DT_Util import html_quote
from OFS.ObjectManager import REPLACEABLE
from OFS.PropertyManager import PropertyManager
from ZPublisher.HTTPRequest import FileUpload
from zLOG import LOG, WARNING
from zExceptions import Unauthorized
from StructuredText import html_with_references

# modules imports
from Utility import getTemplateFile
from stripogram import html2safehtml, html2text
from Const import *

_marker = []

max_img_width = 256
max_img_height = 256

class FormRenderer:
    security = ClassSecurityInfo()
    _valid_tags = ('b', 'a', 'i', 'br', 'p', 'h3', 'ul', 'li', 'font', 'br')
    _image_mime_types = ["image/gif", "image/jpg", "image/jpeg", "image/png"]

    security.declareProtected(view, 'renderFieldStatic')
    def renderFieldStatic(self, ob, field, default='', custom_getattr=None, ignore_unauthorized=1):
        """ render a value from <ob> associated with <field> as string
            optionally use custom_getattr to get at the value
            use this to display data in a report
            may return html (don't html_quote it!)
        """
        return self.field_repr(ob, field,
                               default=default,
                               custom_getattr=custom_getattr,
                               ignore_unauthorized=ignore_unauthorized)
    
    security.declareProtected(view, 'getEditableValue')
    def getEditableValue(self, ob, field, default='', custom_getattr=None):
        attr = self.get_field_name(field)
        ob = getattr(ob, 'aq_base', ob)
        
        v = self.get_value(ob, attr, default, custom_getattr)
        if hasattr(v, 'filename'):
            v = v.filename
        if callable(v):
            try:
                v = v()
            except:
                pass
        return v

    security.declarePrivate('isFieldValidForOb')
    def isFieldValidForOb(self, ob, field, custom_getattr=None, omit_empty=1):
        """ checks whether custom_getattr can get an attribute corresponding to field from ob """
        attr = self.get_field_name(field)
        try:
            v = self.get_value(ob, attr, custom_getattr)
        except AttributeError:
            return 0
        if v or not omit_empty:
            return 1
        return 0
    
    security.declareProtected(view, 'getFieldsValidForOb')    
    def getFieldsValidForOb(self, ob, fields=None, omit_empty_fields=0, omit_hidden_fields=1):
        ''' get fields from a formulator form for which an attribute exists in ob
            if no list of fields is passed in, use all fields from the formulator from.
            if omit_empty_fields: only return fields with corresponding values not None in ob
            if omit_hidden_fields: do not return hidden fields
        '''
        
        def field_is_valid_for_ob(field, get_field_name=self.getFieldName, ob=ob):
            if hasattr(ob, get_field_name(field)):
                return 1
            return 0
        
        def ob_has_value_for_field(field, get_field_name=self.getFieldName, ob=ob):
            if getattr(ob, get_field_name(field), None) is not None:
                return 1
            return 0
        
        if fields is None:
            fields = self.getForm().get_fields()
        fields = filter(field_is_valid_for_ob, fields)
        if omit_hidden_fields:
            fields = filter(lambda x: not x.get_value('hidden'), fields)
        if omit_empty_fields:
            fields = filter(field_has_value_for_ob, fields)
        return fields

    security.declareProtected(view, 'vacuum_request')
    def vacuum_request(self, REQUEST):
        """ remove FileUploadFields from REQUEST """
        for dict in [REQUEST.form, REQUEST.other]:
            for key, value in dict.items():
                if isinstance(value, FileUpload):
                    del dict[key]
        return REQUEST
            
    security.declareProtected(view, 'get_field_name')
    def get_field_name(self, field):
        if field.has_value('alternate_name'):
            alternate_name = field.get_value('alternate_name')
        return alternate_name or field.id

    security.declarePrivate('field_repr')
    def field_repr(self, ob, field, default='', custom_getattr=None, ignore_unauthorized=1):
        ''' return a string represantation of field for ob '''
        def allowed_img_size(width, height, max_width=max_img_width, max_height=max_img_height):
            factor = 1
            if not (width or height):
                return 0, 0
            
            if width > height:
                if width > max_width:
                    factor = max_width / float(width)
            else:
                if height > max_height:
                    factor = max_height / float(height)
            return tuple(map(int, [width * factor, height * factor]))

        # if ob is an acquisition wrapper, get an unwrapped version in base_ob
        base_ob = getattr(ob, 'aq_base', ob)
        
        field_name = self.get_field_name(field)
        field_value = self.get_value(base_ob, field_name, default=default, custom_getattr=custom_getattr,
                                ignore_unauthorized=ignore_unauthorized)
        if field_value is None:
            return default
        meta_type = field.meta_type
        if meta_type == "StringField":
            return html_quote(field_value)
        elif meta_type == "PasswordField":
            return "*" * 8
        elif meta_type == "EmailField":
            return """<a href="mailto:%s">%s</a>""" % (url_quote(field_value), html_quote(field_value))
        elif meta_type == "CheckBoxField":
            if field_value:
                return "Yes"
            else:
                return "No"
        elif meta_type == "IntegerField":
            return  str(field_value)
        elif meta_type == "RangedIntegerField":
            return  str(field_value)
        elif meta_type == "FloatField":
            return  str(field_value)
        elif meta_type == "TextAreaField":
            attr = "%s_encoding" % field_name
            encoding = getattr(ob, attr, None)
            if encoding:
                return self.render_text(field_value, encoding)
            else:
                return html_quote(field_value)
        elif meta_type == "LinesField":
            return html_quote(" ".join(field_value))
        elif meta_type == "ListField":
            items = field.get_value("items")
            for value in items:
                if TupleType is type(value) or ListType is type(value):
                    if value[1] == field_value:
                        return value[0]
                else:
                    break
            return html_quote(str(field_value))
        elif meta_type == "RadioField":
            return  html_quote(str(field_value))
        elif meta_type == "DateTimeField":
            return  html_quote(str(field_value))
        elif meta_type in ["MultiCheckBoxField", "MultiListField"]:
            return  html_quote("\n".join(field_value))
        elif meta_type == "LinkField":
            return "<a href='%s' target='_BLANK'>%s</a>" % (url_quote(field_value), html_quote(field_value))
        elif meta_type == "FileField":
            if hasattr(ob, 'absolute_url'):
                base_url = url_quote(ob.absolute_url())
            else:
                base_url = './'
            if hasattr(field_value, 'filename'):
                content_type = field_value.getContentType()
                content_type = content_type.split(";")[0]
                file_url = url_quote(field_value.filename())
                if content_type in self._image_mime_types:
                    orig_width = getattr(field_value, 'width', 0)
                    orig_height = getattr(field_value, 'height', 0)
                    width, height = allowed_img_size(orig_width, orig_height)
                    if width and height:
                        size_attr = " width='%i' height='%i'" % (width, height)
                    else:
                        size_attr = ""
                    result = "<img src='%s/%s'%s border='0' alt='%s'>" \
                             % (base_url, file_url, size_attr, field_value.filename())
                    if orig_width > width:
                        return "<a href='%s/%s' target='_BLANK'>%s</a>" \
                                 % (base_url, file_url, result)
                else:
                    return "<a href='%s/%s'>%s (%s Kb)</a>" % \
                             (base_url, file_url, html_quote(field_value.filename()), field_value.file_kbytes())
            else:
                return html_quote(str(field_value))
        else:
            return "%s (unknown meta_type %s for field %s)" % (html_quote(field_value), meta_type)
            
    security.declareProtected(view, 'get_value')
    def get_value(self, ob, attr, default=_marker, custom_getattr=None, ignore_unauthorized=1):
        """ try to get a value from ob for field using custom_getattr
            if custom_getattr is None, try several hopefully secure methods to
            get at a value (getProperty, dict.get).
        """
        if custom_getattr is None:
            if DictType is type(ob):
                custom_getattr = ob.get
            elif InstanceType is type(ob) and issubclass(ob.__class__, PropertyManager):
                custom_getattr = ob.getProperty
            else:
                custom_getattr = guarded_getattr

        t = type(custom_getattr)
        bound = 0
        # check for bound methods
        if getattr(custom_getattr, 'im_self', None) or getattr(custom_getattr, '__self__', None):
            bound = 1
        try:
            if bound:
                v = custom_getattr(attr)
            else:
                v = custom_getattr(ob, attr)
            return v
            #return callable(v) and v() or v
        except Unauthorized:
            # don't raise an error for properties not allowed for current user
            # import traceback
            # traceback.print_exc()
            # print "unauthorized:", attr
            if ignore_unauthorized:
                return "*"
            raise
        except AttributeError:
            # ignore AttributeErrors
            if default is _marker:
                raise
            return default
    
    security.declareProtected(view, 'render_text')
    def render_text(self, text, format):
        format = format.upper()
        if not text:
            return ''
        if StringType is type(text):
            text = text.split("\n")
        if format==MSG_ENC_PLAIN:
            return "<br />\n".join(map(html_quote,text))
        elif format==MSG_ENC_HTML:
            return "\n".join(text)
        elif format==MSG_ENC_STX:
            text = "\n".join(text).replace('\r', '')
            text = html_with_references(text, level=3)[stx_min:stx_max]
            return text
        return 'Invalid Format!'
    
    render_text__replaceable__ = REPLACEABLE
    
    security.declareProtected(view, 'striptags')
    def striptags(self,s):
        # ''' removes char sequences that invalidate RSS syntax'''     
        return tagRegex.sub("",s)

    security.declareProtected(view, 'html2text')
    def html2text(self,s):
        return html2text(s)
     
    security.declareProtected(view, 'html2safehtml')
    def html2safehtml(self,text):
        valid_tags = getattr(self, 'valid_tags', self._default_valid_tags)
        return strip(html2safehtml(text,self.getProperty('valid_tags',)))

InitializeClass(FormRenderer)

# FormRendererInstance = FormRenderer()

# this snippet taken from Squishdot/Utility.py
#
# figure out what crap STX is gonna throw at us
crap = str(html_with_references('', level=3))
if   crap=='<html>\n<body>\n</body>\n</html>\n':
    stx_min=14
    stx_max=-16
elif crap=='<p>\012<TABLE BORDER=1 CELLPADDING=2>\012</TABLE></p>\012\012':
    stx_min=0
    stx_max=-2
else:
    stx_min=0
    stx_max=100000
    LOG('FormRenderer',
        WARNING,
        'STX has decided to throw out unexpected crap:',
        crap)
