# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports
from urllib import quote as url_quote
from types import TupleType

# zope imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from OFS.Folder import Folder
from OFS.ObjectManager import BadRequestException
from OFS.ObjectManager import checkValidId
from OFS.ObjectManager import REPLACEABLE
from AccessControl.Permissions import view, view_management_screens

# foreign product imports
from Products.OrderedFolder.OrderedFolder import OrderedFolder
from Products.Formulator.Form import FormValidationError

# product imports
from FormRenderer import FormRenderer
from Renderable import Renderable
from Containers import ComponentManager
from Utility import getTemplateFile
from Const import *
from Permissions import *


manage_addFormulatorLayoutForm = getTemplateFile('manage_addFormulatorLayoutForm', globals())
def manage_addFormulatorLayout(self, id, title='', form_id='', REQUEST=None):
    '''manage_addFormulatorLayout(self, id, title='', form_id='', REQUEST=None) -->
       add FormulatorLayout to FormFolder;
       return None or HTML'''

    form = getattr(self, form_id, None)
    if form is None:
        raise # FIXME: return a Message
    ob  = FormulatorLayout(id, title, form_id)
    self._setObject(ob.id, ob)
    ob = self._getOb(ob.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

class FormulatorLayout(FormRenderer, ComponentManager, Renderable):
    """ class FormulatorLayout - provide layout for formulator forms
        FormulatorLayout objects live in FormFolder objects
    """
        
    security = ClassSecurityInfo()
    selected_meta_types = ('Page Template',)
    meta_type = "Formulator Layout"
    icon = 'misc_/FormulatorLayout/FormulatorLayout'
    # Metatypes of fields that can be used more than once in a layout
    # ('Button Field' and 'Label Field' are extensions not available in a plain Formulator distribution)
    REUSABLE_FIELD_TYPES = ('Button Field', 'Label Field') 
    

    manage_options=(
        ComponentManager.manage_options +
        (
        {'label':'Maintenance', 'action':'manage_maintenance'},
        )
        )

    security.declareProtected(ManageFormFolders, 'manage_maintenance')
    manage_maintenance = getTemplateFile('dtml/manage_maintenance')
    
    def __init__(self, id, title, form_id):
        self.id = id
        self.title = title
        self.form_id = form_id
        
    security.declareProtected(view, 'getForm')
    def getForm(self):
        try:
            return self.aq_parent[self.form_id]
        except:
            return None

    security.declareProtected(view, 'getCurrentPage')
    def getCurrentPage(self, template_type):
        pages = self.getValidComponents(template_type)
        if pages:
            return pages[0] # FIXME: really return current page to support multipage forms
        return None

    security.declareProtected(view, 'getRequestOption')
    def getRequestOption(self, key, default=None):
        REQUEST = self.REQUEST
        options = REQUEST.get(FL_OPTIONS_KEY, {})
        return options.get(key, default)

    security.declareProtected(view, 'getCaller')
    def getCaller(self):
        ''' return the object that initially called render() on this formulator layout.
            By default, this is the PUBLISHED object. Set 'caller' in REQUEST to use
            a different object (this can be useful when doing validation)
        '''
        REQUEST = self.REQUEST
        caller = REQUEST.get('caller', None)
        if caller is None:
            try:
                caller = REQUEST['PUBLISHED']
            except Unauthorized:
                # this should never happen
                # assert 0 
                caller = self
        return caller

    security.declareProtected(view, 'getAction')
    def getAction(self):
        ''' return a tuple of (action_title, action) for current form page
            return form.action only for the last page, '.' else.
        '''
        action = self.getRequestOption('form_action')
        if action is None:
            return (self.getForm().title_or_id(), self.getForm().action)
        elif TupleType is type(action):
            return action
        else:
            return ("Submit", action)

    security.declareProtected(view, 'render')
    def render(self, ob=None, custom_getattr=None, form_action=None, after_edit=None, backend=BACKEND_HTML, **kw):
        """ render formulatorlayout for editing (w/ form fields)
            ob: an object to edit
            custom_getattr: a method to get attributes from ob. can be a bound or an unbound method.
                            should be security aware
            form_action: string or tuple. when a string is given, it is used as the 'action' attribute of the form element
                         when a tuple is given, it should be (<action_title>, <action>).
                         <action_title> will be used as the title of the submitbutton,
                         <action> will be used as the 'action' attribute of the form element.
            after_edit: this will be passed as a hidden field to the target of this form's action as a 'call_back'.
                        if given, it should be the id of a method that's in this form's targets acquisition context. 
        """
        REQUEST = self.REQUEST
        current_page = self.getCurrentPage(TEMPLATE_TYPE_DEFAULT)
        self.setRequestOptions(ob=ob, custom_getattr=custom_getattr, form_action=form_action, after_edit=after_edit,
                               backend=backend)
        if current_page:
            return current_page.render()
        else:
            return "-"
    
    security.declareProtected(view, 'renderReport')
    def renderReport(self, ob, custom_getattr=None, backend=BACKEND_HTML, **kw):
        self.setRequestOptions(ob=ob, custom_getattr=custom_getattr, backend=backend)
        result = []
        pages = self.getValidComponents(TEMPLATE_TYPE_REPORT)
        for page in pages:
            result.append(page.renderReport())
        return "\n".join(result)

    security.declareProtected(ManageFormFolders, 'getUnusedFields')
    def getUnusedFields(self):
        all_fields = self.getForm().get_fields()
        used_ids = self.getFieldIds()
        used_fields = filter(lambda x, used_ids=used_ids: x.id in used_ids, all_fields)
        used_fields = filter(lambda x, dup_types=self.REUSABLE_FIELD_TYPES: x.meta_type not in dup_types, used_fields)
        unused_fields = filter(lambda x, l=used_fields: x not in l, all_fields)
        return unused_fields

    security.declareProtected(ManageFormFolders, 'manage_removeBrokenFieldComponents')
    def manage_removeBrokenFieldComponents(self, REQUEST=None):
        ''' remove field components without a corresponding field in our assiciated formulator object '''
        components = self.getBrokenFieldComponents()
        containers = map(lambda x: x.aq_parent, components)
        for component, container in zip(components, containers):
            container.manage_delObjects([component.id])
        if REQUEST:
            component_ids = map(lambda x:x.id, components)
            if components:
                message = "%s component%s removed (%s)" % (len(components),
                                                           (len(components) != 1) and 's' or '',
                                                           ', '.join(component_ids)
                                                           )
            else:
                message = 'no broken components'
            return self.manage_maintenance(manage_tabs_message=message)

    security.declareProtected(ManageFormFolders, 'edit')
    def manage_edit(self, form_id, REQUEST):
        ''' edit this formulator layout
            use this the change to associated form
        '''
        self.form_id = form_id
        message = 'My associated form is now %s.' % form_id
        return self.manage_maintenance(manage_tabs_message=message)

    security.declareProtected(ManageFormFolders, 'getAvailableForms')
    def getAvailableForms(self):
        return self.aq_parent.objectValues('Formulator Form')

    security.declareProtected(ManageFormFolders, 'test')
    def test(self, REQUEST):
        ''' test this formulator layout '''
        try:
            test_results = self.getForm().validate_all(REQUEST)
        except FormValidationError, error_value:
            test_results = {}
            REQUEST.set('error_value', error_value)
        return self.index_html(REQUEST=REQUEST, test_results=test_results)

    security.declareProtected(ManageFormFolders, 'index_html')
    index_html = getTemplateFile('templates/testFormulatorLayout', globals=globals())
    
        
    security.declarePrivate('getBoundNames')
    def getBoundNames(self, REQUEST, template_type=TEMPLATE_TYPE_DEFAULT, **kw):
        form = self.getForm()
        components = self.getValidComponents(template_type)
        return {'form': form, 'components': components}
                             
    security.declarePrivate('setRequestOptions')
    def setRequestOptions(self, **kw):
        REQUEST = self.REQUEST
        if not REQUEST.has_key(FL_OPTIONS_KEY):
            options = kw
            REQUEST.set(FL_OPTIONS_KEY, options)
        else:
            options = REQUEST.get(FL_OPTIONS_KEY)
            options.update(kw)

    security.declarePublic('getAlignments')
    def getAlignments(self):
        return ALIGNMENTS
        
    security.declarePublic('getLabelTypes')
    def getLabelTypes(self):
        return LABEL_TYPES
        
    security.declarePublic('getButtonTypes')
    def getButtonTypes(self):
        return BUTTON_TYPES
        
    security.declarePublic('getTemplateTypes')
    def getTemplateTypes(self):
        return TEMPLATE_TYPES

    def getBackendTypes(self):
        return BACKEND_TYPES
    
InitializeClass(FormulatorLayout)
